/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef ALIGN_HH
#define ALIGN_HH

#include "config.h"
#include <cstddef>
#include <cstdlib>

namespace SharpConstruct
{
	template< typename T > class Align
	{
	public:
		typedef T value_type;
		typedef value_type* pointer;
		typedef const value_type* const_pointer;
		typedef value_type& reference;
		typedef const value_type& const_reference;
		typedef size_t size_type;
		typedef std::ptrdiff_t difference_type;

		template< typename U >
		struct rebind
		{
			typedef Align< U > other;
		};

		pointer allocate( size_type n )
		{
			void* mem;
#ifdef HAVE_POSIX_MEMALIGN
			posix_memalign( &mem, __alignof( value_type ), n * sizeof( value_type ) );
#else
#ifdef _MSC_VER
			mem=_aligned_malloc(n * sizeof( value_type ), __alignof( value_type ));
#else
#error Fixme
#endif // _MSC_VER
#endif // HAVE_POSIX_MEMALIGN
			return reinterpret_cast< pointer >( mem );
		}
		inline void deallocate( pointer p, size_type )
		{ 
#ifdef HAVE_POSIX_MEMALIGN
			free( p );
#else
#ifdef _MSC_VER
			_aligned_free( p );
#else
#error Fixme
#endif // _MSC_VER
#endif // HAVE_POSIX_MEMALIGN
		}

		inline void construct( pointer p, const T& t )
		{
			new( p ) T( t );
		}
		inline void destroy( pointer p )
		{
			p->~T();
		}

		size_type max_size() const
		{
			size_type _Count = (size_type)(-1) / sizeof (value_type);
			return (0 < _Count ? _Count : 1);
		}

                bool operator==(const Align<T>& r) const
                {
                  return true;
                }

	};
}

#endif
