/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "DeformationBase.hh"

#ifdef WIN32
#include <windows.h>
// For lround() replacement
#include "win32.h"
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <cmath>

using SharpConstruct::DeformationBase;

DeformationBase::DeformationBase()
: over_( NONE ), selected_( NONE )
{}

DeformationBase::~DeformationBase()
{}

void DeformationBase::FlatDraw( const Gdk::Point& mloc,
				const Gdk::Point& model_center )
{
	model_center_ = model_center;

	if( over_ != NONE )
		return;

	glLineWidth( 2 );
	glEnable( GL_LINE_STIPPLE );
	const GLushort dash( 43690 );
	glLineStipple( 3, dash );
	glBegin( GL_LINES );
	glColor3f( 0, 0, 0 );
	glVertex3f( model_center.get_x(), model_center.get_y(), 0 );
	glVertex3f( mloc.get_x(), mloc.get_y(), 0 );
	glEnd();
	glLineWidth( 1 );
	glDisable( GL_LINE_STIPPLE );
}

void DeformationBase::Draw( const Gdk::Point& mloc )
{
	unsigned names[ 512 ]; // Here's a nice buffer overflow waiting to happen...
	unsigned sel_count( 0 );
	glSelectBuffer( 512, names );
	glRenderMode( GL_SELECT );
	
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();

	int viewport[ 4 ];
	glGetIntegerv( GL_VIEWPORT, viewport );
	gluPickMatrix( mloc.get_x(), mloc.get_y(),
		       4, 4, viewport );
	glOrtho( 0, viewport[ 2 ], 0, viewport[ 3 ], -200, 200 );
	glMatrixMode( GL_MODELVIEW );
	draw_and_select_();

	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );

	sel_count = glRenderMode( GL_RENDER );

	glLineWidth( 2 );
	draw_and_select_();
	glLineWidth( 1 );

	// Now process the first name:
	if( sel_count > 0 )
		over_ = ( Axis )names[ 3 ];
	else
		over_ = NONE;
}

void DeformationBase::MouseDown( const Gdk::Point& mloc )
{
	selected_ = over_;
	if( over_ == NONE )
		selected_ = ALL;
	mouse_down_loc_ = mloc;
}

void DeformationBase::MouseMove( const Gdk::Point& )
{}

void DeformationBase::draw_and_select_()
{
	glPushMatrix();
	glScalef( 10, 10, 10 );

	float xcol, ycol, zcol;
	xcol = ycol = zcol = 0.5;
	if( over_ == X )
		xcol = 1;
	else if( over_ == Y )
		ycol = 1;
	else if( over_ == Z )
		zcol = 1;

	glInitNames();

	glPushName( X );
	glColor3f( xcol, 0, 0 );
	draw_handle_( X, 1, 0, 0 );

	glLoadName( Y );
	glColor3f( 0, ycol, 0 );
	draw_handle_( Y, 0, 0, 1 );

	glLoadName( Z );
	glColor3f( 0, 0, zcol );
	draw_handle_( Z, 0, 1, 0 );
	glPopName();

	glPopMatrix();
}

Gdk::Point SharpConstruct::DeformationBase::project_2d_( const Point3D& in ) const
{
	double model_matrix[ 16 ];
	double projection_matrix[ 16 ];
	int viewport[ 4 ];
	double x, y, z;

	glGetDoublev( GL_PROJECTION_MATRIX, projection_matrix );
	glGetIntegerv( GL_VIEWPORT , viewport );
	glGetDoublev( GL_MODELVIEW_MATRIX, model_matrix );
	gluProject( in.X(), in.Y(), in.Z(),
		    model_matrix, projection_matrix, viewport, 
		    &x, &y, &z );

	return Gdk::Point( lround( x ), lround( y ) );
}
